<?php

namespace App\Controller\Admin;

use App\Entity\Employee;
use Doctrine\DBAL\Types\BooleanType;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\QueryBuilder;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\BooleanField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ChoiceField;
use EasyCorp\Bundle\EasyAdminBundle\Field\EmailField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

class EmployeeCrudController extends AbstractCrudController
{
    public function __construct(
        private UserPasswordHasherInterface $userPasswordHasher,
    ) {
    }

    public static function getEntityFqcn(): string
    {
        return Employee::class;
    }
    public function configureCrud(Crud $crud): Crud
    {
        return $crud
            ->setSearchFields(['lastname','firstname'])
            ->setPageTitle('index', 'Liste des employés')
            ->setEntityLabelInSingular('cet employé')
            ->setEntityLabelInPlural('Employés')
            ->setDateTimeFormat('dd/MM/yyyy à HH:mm')
            ;
     }

    public function configureFields(string $pageName): iterable
    {

        if ($this->isGranted('ROLE_SUPER_ADMIN')){
            return [
                AssociationField::new('company','Entreprise')->setPermission('ROLE_SUPER_ADMIN'),
                //TextField::new('password', 'Mot de passe')->onlyOnForms(),
                TextField::new('lastname','Nom'),
                TextField::new('firstname','Prénom'),
                EmailField::new('email','Email')->setPermission('ROLE_ADMIN'),
                ChoiceField::new('roles','Permissions')
                    ->setChoices([
                        'Super Administrateur'=>'ROLE_SUPER_ADMIN',
                        'Administrateur' => 'ROLE_ADMIN',
                        'Editeur' =>'ROLE_EDITOR',
                        'Utilisateur' =>'ROLE_USER',
                    ])
                    ->setRequired(false)
                    ->allowMultipleChoices(true),
                TextField::new('phone','Téléphone'),
                BooleanField::new('isVerified','Vérification'),
                TextField::new('plainPassword', 'Mot de passe')->onlyOnForms(),
            ];
        }elseif ($this->isGranted('ROLE_ADMIN')) {
            return [
                TextField::new('lastname','Nom'),
                TextField::new('firstname','Prénom'),
                EmailField::new('email','Email')->setPermission('ROLE_ADMIN'),
                ChoiceField::new('roles','Permissions')
                    ->setChoices([
                        'Administrateur' => 'ROLE_ADMIN',
                        'Editeur' =>'ROLE_EDITOR',
                        'Utilisateur' =>'ROLE_USER',
                    ])
                    ->setRequired(false)
                    ->allowMultipleChoices(true),
                TextField::new('phone','Téléphone'),
                TextField::new('plainPassword', 'Mot de passe')->onlyOnForms(),
            ];
        }else{
            return [
                TextField::new('lastname','Nom'),
                TextField::new('firstname','Prénom'),
                TextField::new('phone','Téléphone'),
                //TextField::new('password', 'Mot de passe')->onlyOnForms(),
                TextField::new('plainPassword', 'Mot de passe')->onlyOnForms(),
            ];
        }

    }
    public function createEntity(string $entityFqcn){
        $user=$this->getUser();
        $company = $user->getCompany();
        
        $employee=new Employee();
        $employee->setCompany($company);
        //$employee->setEmployee($employee);
        $employee->setCreatedAt(new \DateTime());
        $employee->setUpdatedAt(new \DateTime());
        return $employee;
    }

    public function persistEntity(EntityManagerInterface $entityManager, $entityInstance):void
    {
        $entityInstance->setPassword($this->userPasswordHasher->hashPassword($entityInstance, $entityInstance->getPlainPassword()));
        $entityManager->persist($entityInstance);
        $entityManager->flush();
    }


    public function configureActions(Actions $actions): Actions
    {
         if ($this->isGranted('ROLE_ADMIN')){
             return $actions;
         }else{
             return $actions->disable(Action::NEW, Action::EDIT, Action::DELETE);
        }
    }

    public function createIndexQueryBuilder(SearchDto $searchDto, EntityDto $entityDto, FieldCollection $fields, FilterCollection $filters): QueryBuilder
    {  
        $user=$this->getUser();
        $companyId = $user->getCompany()->getId();
        $queryBuilder=parent::createIndexQueryBuilder($searchDto, $entityDto, $fields, $filters);
        if (!$this->isGranted('ROLE_SUPER_ADMIN')){
            $queryBuilder->where('entity.company = (:company_0)')
                        ->setParameter('company_0', $companyId);
        }
        $queryBuilder->orderBy('entity.lastname', 'ASC')
                    ->addOrderBy('entity.firstname', 'ASC');
        return $queryBuilder;
    }

}